<?php
# This file is distributed under GNU GPLv2 and further versions.
# This file was written by Zdenek Styblik in co-operation with Tarmo Hywarinen
#
# Zdenek Styblik
# e-mail: zdenek <dot> styblik <at> gmail <dot> com
#
# Last update: 2008-07-07
#
# Notes: stripped out and mostly cleaned up
	
class defaultLib {

	public $scriptname;
	// *pure* xml
	public $xmloutput;
	// xsl
	public $template;
	// xsl files to import
	public $import;
	// debug option
	public $debug;
	// config file path
	public $configfile;
	// main page layout
	private $layout;
	private $moduleClass;

	function __construct($path = NULL)
 	{
		define('PATH', $path);
		// default module to load
		define('MODULE', "index");
		// if no HTTP_HOST set, hostname to add
		define('HTTPHOST', "");
		# we *have to* start xml with root tag
		$this->xmloutput = "<root>\n";
		# default page layout
		$this->layout = "layout.xsl"; // NULL==XML out only;
		# we'll use this later
		$this->import = new StdClass;
		// DEBUG On/Off
		$this->debug = 0;
	}
	function __destruct()
 	{
		$this->xmloutput;
		$this->import;
		$this->layout;
		$this->template;
	}
	public function start()
 	{
		session_start();
		$this->scriptname = $this->getScriptname();
		try 
		{
			$module = MODULE;
			# if $_GET['module'] is empty, then load "default" module.
			if (isset($_GET['module']) && !empty($_GET['module']) && !is_null($_GET['module']))
			{
				$module = $this->stripMe($_GET['module']);
			}
			# if $module.php exists, then proceed, else set default module
			if (!file_exists("{$this->getPath()}modules/{$module}.php"))
			{
				$module = MODULE;
			}
			// not even default module is readable?
			if (!file_exists("{$this->getPath()}modules/{$module}.php"))
				throw new Exception("Unable to open default module '{$this->getPath()}modules/{$module}.php'. Terminated.");
			$this->setContent($module);
			if (property_exists($this->moduleClass, 'settings'))
			{
				$this->handleSettings($this->moduleClass->settings);
			}
			if (!is_null($this->layout))
			{
				if (file_exists("{$this->getPath()}xsl/layout/{$this->layout}"))
			 	{
					$this->template = $this->xslPrepare($this->import, "root", "{$this->getPath()}xsl/layout/{$this->layout}");
					header("Content-type: text/html; charset=utf-8");
					# enclose XML <root> tag, el Stupido
					$this->xmloutput.= "</root>\n";
					print $this->transform($this->xmloutput, $this->template);
				}
				else
			 	{
					# Layout is nowhere to find. What do you want to do? Sit in the corner and cry! :)
					throw new Exception("Layout({$this->getPath()}xsl/layout/{$this->layout}) not found.\n This means critical error.");
				}
			}
			else
			{
				$this->xmloutput.= "</root>\n";
				header("Content-type: text/xml; charset=utf-8");
				echo "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n";
				echo $this->xmloutput;
			}
		}
	 	catch (Exception $e)
		{
			echo "Caught exception: ", $e->getMessage(), ";\n";
		}
	}
	# set content
	private function setContent($module) {
		require_once("{$this->getPath()}modules/{$module}.php");
		$this->moduleClass = new $module;
		$this->moduleClass->init();
	}

	# handle settings from module
	private function handleSettings($input = null) {
		if (!is_null($input) && is_object($input)) {
			if (property_exists($input, "layout")) {
				$this->layout = $input->layout;
			}
			if (property_exists($input, "import")) {
				$this->import = $input->import;
			}
			if (property_exists($input, "xmlobject")) {
				$this->xmloutput.= $this->objToXML($input->xmlobject);
			}
		} else {
			# dummy
		}
	}
	### Xml / Xsl ###
	# desc: prepares main page xsl layout with css/imports/etc.
	# $import:
	# $template:
	# $file:
	# @return: string(well formed xsl);
	public function xslPrepare($import = null, $templatename = null, $file) {
		if (is_null($templatename))
			die("xslPrepare: template name can't be NULL!");
		$xsl = "";
		if (!is_null($import) && !is_null($templatename))
	 	{
			$xsl = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n<xsl:stylesheet version=\"1.0\" xmlns:xsl=\"http://www.w3.org/1999/XSL/Transform\">\n";
			foreach ($import as $k=>$v)
		 	{
				if (file_exists($v))
			 	{
					$xsl.= "<xsl:import href=\"{$v}\" />\n";
				}
			}
			$xsl.= "<xsl:output method=\"xml\" encoding=\"utf-8\" indent=\"yes\" doctype-public=\"-//W3C//DTD XHTML 1.0 Transitional//EN\" doctype-system=\"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\"/>";
			$xsl.= "<xsl:template match=\"{$templatename}\" name=\"{$templatename}\">\n";
			if (file_exists($file))
		 	{
				$xsl.= file_get_contents($file);
			}
			$xsl.= "</xsl:template>\n</xsl:stylesheet>";
		}
		else
	 	{
			$xsl = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n<xsl:stylesheet version=\"1.0\" xmlns:xsl=\"http://www.w3.org/1999/XSL/Transform\">\n";
			$xsl.= "<xsl:output method=\"xml\" encoding=\"utf-8\" indent=\"yes\" doctype-public=\"-//W3C//DTD XHTML 1.0 Transitional//EN\" doctype-system=\"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\"/>";
			$xsl.= "<xsl:template match=\"{$templatename}\" name=\"{$templatename}\">\n";
			if (file_exists($file))
		 	{
				$xsl.= file_get_contents($file);
			}
			$xsl.= "</xsl:template>\n</xsl:stylesheet>";
		}
		return $xsl;
	}
	# desc: converts php object into xml tree
	# $obj: std Class;
	# @return: string;
	public function objToXML($obj, $parent = null, $deep = 1)
 	{
		$xmloutput = "";
		if (is_object($obj) || is_array($obj))
	 	{
			foreach ($obj as $k=>$v)
		 	{
				if (is_object($v))
			 	{
					if (is_numeric($k))
				 	{
						$xmloutput.= self::objToXML($v, $parent, $deep+1);
					}
					else
				 	{
						$xmloutput.= str_repeat("\t", $deep);
						$xmloutput.="<{$k}>\n";
						$xmloutput.= self::objToXML($v, $k, $deep+1);
						$xmloutput.= str_repeat("\t", $deep);
						$xmloutput.="</{$k}>\n";
					}
				}
				elseif (is_array($v))
			 	{
					foreach($v as $c=>$d)
				 	{
						$xmloutput.= str_repeat("\t", $deep);
						$xmloutput.="<{$k}>\n";
						$xmloutput.= self::objToXML($d, $parent, $deep+1);
						$xmloutput.= str_repeat("\t", $deep);
						$xmloutput.="</{$k}>\n";
					}
				}
				else
			 	{
					for ($a =0; $a < $deep; $a++)
				 	{
						$xmloutput.= "\t";
					}
					$xmloutput.= "<{$k}>{$v}</{$k}>\n";
				}
			}
		}
		return $xmloutput;
	}
	# $xml: string(well formated xml);
	# $xsltemplate: string(well formated xsl);
	# @return: string(xhtml);
	public function transform($xml, $xsltemplate)
 	{
		### MANUAL DEBUG ###
		#echo $this->xmloutput; exit();
		#echo $this->template; exit();
		# load XSL file
		$xsl = new DomDocument();
		$xsl->loadXML($xsltemplate);
		# load XML file
		$inputdom = new DomDocument();
		$inputdom->loadXML($xml);
		#	load XsltProcessor and fed it with XSL file
		$proc = new XsltProcessor();
		$xsl = $proc->importStylesheet($xsl);
		# transform XML + XSL --> XHTML and do output
		$newdom = $proc->transformToDoc($inputdom);
		$newdom->formatOutput = true;
		$retval = $newdom->saveXML();
		$retval = ereg_replace(' xmlns=""', '', $retval);
		return $retval;
	}

	public function replacer($input, $tag) {
		$replaced = $input;
		if ( is_array($tag) ) {
			foreach ($tag  as $key=>$value) {
				$taglen = strlen($key);
				$fullTag = "{" . $key . "}";
				$replacement = $value;
				$replaced = ereg_replace($fullTag, "{$value}", $replaced);
			}
		}
		return $replaced;
	}
		
	### ENDo Xsl ###
	
	### Common ###
	# add slashes with \n -> <br /> conv.
	# var: string
	public function stripMe($string, $forcequote = 0, $replacebr = 1)
 	{
		if(!isset($string))
	 	{
	 	  $string="";
		}
		$string = trim($string);
		if($forcequote == 1)
	 	{
	 	  $string = addslashes($string);
		}
		elseif (!get_magic_quotes_gpc() && $forcequote == 0)
	 	{
	 		$string=addslashes($string);
		}
		if ($replacebr == 1)
	 	{
			$string=ereg_replace("\n","<br />", $string);
		}
	 	return $string;
	}
	# turn debug on/off
	# $int: bit;
	# @return: void;
	public function setDebug($int)
 	{
		if ($int == 1)
	 	{
			$this->debug = 1;
		}
		else
	 	{
			$this->debug = 0;
		}
	}
	# desc: returns path to the files
	# @return: string
	protected function getPath()
 	{
		return PATH;
	}
	# desc: returns debug state
	# @return: bit;
	private function getDebugState()
	{
		return $this->debug;
	}
	public function getHostname()
	{
		if (isset($_SERVER['HTTP_HOST']))
		{
			return $_SERVER['HTTP_HOST'];
		}
		else
		{
			return HTTPHOST;
		}
	}
	# return the name of this script
	# var: none
	# todo: http/https
	public function getScriptname()
 	{
		if (isset($_SERVER['HTTP_HOST']))
	 	{
			$host = "http://" . $_SERVER["HTTP_HOST"];
		}
		else
	 	{
			$host = HTTPHOST;
		}
		$returnMe = $host . $_SERVER['PHP_SELF'];
		return $returnMe;
	}
	### ENDo Common ###
}
?>
